from django.conf import settings
from django.shortcuts import render
from django.http import JsonResponse
from django.core.exceptions import ObjectDoesNotExist
import json

from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_GET, require_POST
from django.views.decorators.csrf import csrf_exempt

from .models import Shape, Stop

# @login_required(login_url='/admin/login/') # TODO login
@login_required # TODO login
@require_GET
def shapeeditor(request):
    routing_machine_url=''
    extent=''
    center=''

    try:
        routing_machine_url = settings.SHAPEEDITOR_ROUTING_MACHINE_URL
    except:
        routing_machine_url = "http://router.project-osrm.org/route/v1/driving/"

    try:
        center = settings.SHAPEEDITOR_MAP_CENTER
    except:
        center = "null"

    try:
        extent = settings.SHAPEEDITOR_MAP_EXTENT_AREA
    except:
        extent = "null"

    context = { # TODO
        "routing_machine_url": routing_machine_url, # TODO
        "extent": extent,
        "center": center
    }

    return render(request, 'shapeeditor/index.html', context)

@login_required # TODO admin only
@require_POST
@csrf_exempt # TODO remove
def push_shapes(request):
    try:
        body = request.body
        data = json.loads(body)

        if 'head' not in data or 'body' not in data:
            if 'points' not in data['body'] or 'distances' not in data['body'] or 'id' not in data['body']:
                return JsonResponse(status=400, data={"message": "Invalid data format"})

        shapes = []
        for item in data["body"]: # for each shape
            shapes.append(Shape(
                shape_id = item['id'],
                lines = item
            ))
        Shape.objects.all().delete()
        Shape.objects.bulk_create(shapes)

        return JsonResponse(status=200, data={"message": "Web push successful"})
    except TypeError:
        return JsonResponse(status=500, data={"message": "An error occurred"})

@login_required # TODO admin only
@require_POST
@csrf_exempt # TODO remove
def push_stops(request):
    try:
        body = request.body
        data = json.loads(body)

        if 'head' not in data or 'body' not in data:
            return JsonResponse(status=400, data={"message": "Invalid data format"})


        stops = []
        for item in data["body"]: # for each stop
            stops.append(Stop(
                stop_id = item['stop_id'],
                lines = item
            ))
        Stop.objects.all().delete()
        Stop.objects.bulk_create(stops)


        return JsonResponse(status=200, data={"message": "Web push successful"})
    except TypeError:
        return JsonResponse(status=500, data={"message": "An error occurred"})


### API JSON ###
def get_stop(element_id):
    try:
        stop = Stop.objects.get(stop_id=element_id)
        return JsonResponse(status=200, data=stop.lines)
    except ObjectDoesNotExist:
        return JsonResponse(status=500, data={"message": "Stop matching query does not exist."})
    except:
        return JsonResponse(status=500, data={"message": "An error occurred"})

def get_shape(element_id): # a single shape
    try:
        shape = Shape.objects.get(shape_id=element_id)
        return JsonResponse(status=200, data=shape.lines)
    except ObjectDoesNotExist:
        return JsonResponse(status=500, data={"message": "Shape matching query does not exist."})
    except:
        return JsonResponse(status=500, data={"message": "An error occurred"})

@require_GET
def api (request, element, element_id):
    print(request)

    if ("shape" == element):
        return get_shape(element_id)
    elif ("stop" == element):
        return get_stop(element_id)

    return JsonResponse(status=500, data={"message": "Wrong element on request"})



from django.db import models

# Create your models here.

def default_stop_line(): # TODO
    return {'id': [], 'points': [], 'distances': []}

def default_shape_line():
    return {'id': [], 'points': [], 'distances': []}

# models

class Stop (models.Model):
    stop_id = models.CharField(
        primary_key=True,  # This makes the id unique in database
        db_index=True,
        max_length=80)

    lines = models.JSONField(default=default_stop_line) # TODO replace with multiple fields

    def __str__(self):
        return self.stop_id

class Shape (models.Model):
    shape_id = models.CharField(
        primary_key=True, # This makes the id unique in database
        db_index=True,
        max_length=80)

    lines = models.JSONField(default=default_shape_line)

    def __str__(self):
        return self.shape_id



#!/usr/bin/env python
# $ pip install twine setuptools wheel

import io
from os import system as bash, path
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

# Package meta-data.
VERSION = '0.1.4'

REQUIRED = [
]

EXTRAS = {
}

here = path.abspath(path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(path.join(here, project_slug, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        bash('echo {0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        bash('echo twine upload dist/*')

        self.status('Pushing git tags…')
        bash('echo git tag v{0}'.format(about['__version__']))
        bash('echo git push --tags')

        sys.exit()

class BuildCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(path.join(here, 'build'))
            rmtree(path.join(here, 'dist'))
        except OSError:
            pass

        print("Run npm run dist first for production version")

        self.status('Building Source and Wheel (universal) distribution…')
        bash('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        sys.exit()

setup(
    name='shapeeditor',
    version=about['__version__'],
    description='GTFS Editor frontend app for Django',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Jeancarlo Hidalgo',
    author_email='jeancahu@gmail.com',
    python_requires='>=3.7.0',
    url='https://github.com/jeancahu/GTFS_shapes_editor_JS',
    package_dir={"": "src"},
    packages=find_packages(where="src", exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    cmdclass={
        'upload': UploadCommand,
        'dist': BuildCommand
    },
)



from django.urls import path

from . import views

urlpatterns = [
    path('', views.shapeeditor, name='shapeeditor'),
    path('push_shapes', views.push_shapes, name='shapeeditor_push_shapes'),
    path('push_stops', views.push_stops, name='shapeeditor_push_stops'),
    path('api/<str:element>/<str:element_id>', views.api, name='shapeeditor_api')
]



# Generated by Django 3.1.7 on 2021-06-10 00:50

from django.db import migrations, models
import shapeeditor.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Shape',
            fields=[
                ('shape_id', models.CharField(db_index=True, max_length=80, primary_key=True, serialize=False)),
                ('lines', models.JSONField(default=shapeeditor.models.default_shape_line)),
            ],
        ),
        migrations.CreateModel(
            name='Stop',
            fields=[
                ('stop_id', models.CharField(db_index=True, max_length=80, primary_key=True, serialize=False)),
                ('lines', models.JSONField(default=shapeeditor.models.default_stop_line)),
            ],
        ),
    ]
